Class: BTC::AssetTransactionBuilder::Result
- Inherits:
-
Object
- Object
- BTC::AssetTransactionBuilder::Result
- Defined in:
- lib/btcruby/open_assets/asset_transaction_builder/result.rb,
lib/btcruby/open_assets/asset_transaction_builder/result.rb
Overview
Internal-only setters.
Instance Attribute Summary collapse
-
#asset_transaction ⇒ Object
AssetTransaction instance with full details about asset issuance and transfer.
-
#assets_cost ⇒ Object
Total cost of all issues and transfers (not including the mining fees and asset change outputs) All of that amount is owned by the asset holders and can be extracted or returned during re-sell.
-
#fee ⇒ Object
Total mining fee for all composed transactions.
-
#transactions ⇒ Object
Array of BTC::Transaction instances.
-
#unsigned_input_indexes ⇒ Object
Array of arrays.
Instance Method Summary collapse
-
#initialize ⇒ Result
constructor
A new instance of Result.
Constructor Details
#initialize ⇒ Result
Returns a new instance of Result.
29 30 31 32 33 34 35 |
# File 'lib/btcruby/open_assets/asset_transaction_builder/result.rb', line 29 def initialize self.transactions = [] self.unsigned_input_indexes = [] self.asset_transaction = nil self.fee = 0 self.assets_cost = 0 end |
Instance Attribute Details
#asset_transaction ⇒ Object
AssetTransaction instance with full details about asset issuance and transfer.
20 21 22 |
# File 'lib/btcruby/open_assets/asset_transaction_builder/result.rb', line 20 def asset_transaction @asset_transaction end |
#assets_cost ⇒ Object
Total cost of all issues and transfers (not including the mining fees and asset change outputs) All of that amount is owned by the asset holders and can be extracted or returned during re-sell.
27 28 29 |
# File 'lib/btcruby/open_assets/asset_transaction_builder/result.rb', line 27 def assets_cost @assets_cost end |
#fee ⇒ Object
Total mining fee for all composed transactions.
23 24 25 |
# File 'lib/btcruby/open_assets/asset_transaction_builder/result.rb', line 23 def fee @fee end |
#transactions ⇒ Object
Array of BTC::Transaction instances.
These may have unsigned inputs and must be published in the order.
The last transaction is wrapped by the asset_transaction.
Typically, this array contains just one transaction. When issuing an asset,
it may contain two transactions.
13 14 15 |
# File 'lib/btcruby/open_assets/asset_transaction_builder/result.rb', line 13 def transactions @transactions end |
#unsigned_input_indexes ⇒ Object
Array of arrays. Each top-level array refers to a list of input indexes to be signed. Some inputs can be signed already.
17 18 19 |
# File 'lib/btcruby/open_assets/asset_transaction_builder/result.rb', line 17 def unsigned_input_indexes @unsigned_input_indexes end |